"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RunnerController = exports.runnerControllerOptionsDefault = void 0;
const child_process_1 = require("child_process");
const crypto_1 = __importDefault(require("crypto"));
const axios_1 = __importDefault(require("axios"));
const get_port_1 = __importDefault(require("get-port"));
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)('deeplinks:runner-controller');
exports.runnerControllerOptionsDefault = {
    gqlURN: 'deep_deeplinks_1:3006',
    network: 'deep_network',
    portsHash: {},
    handlersHash: {},
    docker: 0
};
class RunnerController {
    constructor(options) {
        this.portsHash = {};
        this.handlersHash = {};
        this.network = (options === null || options === void 0 ? void 0 : options.network) || exports.runnerControllerOptionsDefault.network;
        this.gqlURN = (options === null || options === void 0 ? void 0 : options.gqlURN) || exports.runnerControllerOptionsDefault.gqlURN;
        this.docker = (options === null || options === void 0 ? void 0 : options.docker) || exports.runnerControllerOptionsDefault.docker;
        this.portsHash = (options === null || options === void 0 ? void 0 : options.portsHash) || exports.runnerControllerOptionsDefault.portsHash;
        this.handlersHash = (options === null || options === void 0 ? void 0 : options.handlersHash) || exports.runnerControllerOptionsDefault.handlersHash;
    }
    ;
    findPort(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const { handler, forcePort, forceName } = options;
            const { network, handlersHash, portsHash, docker, gqlURN } = this;
            console.log({ handlersHash, portsHash });
            const containerName = forceName || crypto_1.default.createHash('md5').update(handler).digest("hex");
            if (handlersHash[containerName])
                return { port: handlersHash[containerName] };
            let dockerPort = forcePort || (yield (0, get_port_1.default)());
            let dockerRunResult;
            let done = false;
            let count = 30;
            while (!done) {
                console.log({ count });
                if (count < 0)
                    return { error: 'timeout findPort' };
                count--;
                try {
                    dockerRunResult = (0, child_process_1.execSync)(`docker run -e PORT=${dockerPort} -e GQL_URN=${gqlURN} -e GQL_SSL=0 --name ${containerName} ${docker ? `--expose ${dockerPort}` : `-p ${dockerPort}:${dockerPort}`} --net ${network} -d ${handler} && npx wait-on http://${docker ? portsHash[dockerPort] : 'localhost'}:${dockerPort}/healthz`).toString();
                    console.log('dockerRunResult', { dockerRunResult });
                }
                catch (e) {
                    dockerRunResult = e.stderr;
                }
                if (dockerRunResult.indexOf('port is already arllocated') !== -1) {
                    continue;
                }
                else if (dockerRunResult.indexOf('is already in use by container') !== -1) {
                    let dockerStopResult = yield (0, child_process_1.execSync)(`docker stop ${containerName} && docker rm ${containerName}`).toString();
                    console.log('dockerStopResult', { dockerStopResult });
                }
                else {
                    done = true;
                    handlersHash[containerName] = dockerPort;
                    portsHash[dockerPort] = containerName;
                }
                if (!done)
                    while (portsHash[dockerPort]) {
                        dockerPort = yield (0, get_port_1.default)();
                    }
            }
            return { port: dockerPort };
        });
    }
    initHandler(options) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const { portsHash, docker } = this;
            const { port } = options;
            let initResult;
            try {
                const initRunner = `http://${docker ? portsHash[port] : 'localhost'}:${port}/init`;
                console.log('initRunner', { initRunner });
                initResult = yield axios_1.default.post(initRunner);
                console.log('initResult', { initResult: initResult.toString() });
            }
            catch (e) {
                console.log('error', e);
                return ({ error: e });
            }
            if ((_a = initResult === null || initResult === void 0 ? void 0 : initResult.data) === null || _a === void 0 ? void 0 : _a.error)
                return { error: (_b = initResult === null || initResult === void 0 ? void 0 : initResult.data) === null || _b === void 0 ? void 0 : _b.error };
            return {};
        });
    }
    callHandler(options) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            const { code, port, data, jwt } = options;
            const { portsHash, docker } = this;
            try {
                const callRunner = `http://${docker ? portsHash[port] : 'localhost'}:${port}/call`;
                console.log('callRunner', { callRunner, params: options });
                const result = yield axios_1.default.post(callRunner, { params: options });
                if ((_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a.error)
                    return { error: (_b = result === null || result === void 0 ? void 0 : result.data) === null || _b === void 0 ? void 0 : _b.error };
                if ((_c = result === null || result === void 0 ? void 0 : result.data) === null || _c === void 0 ? void 0 : _c.resolved) {
                    return result.data.resolved;
                }
                return Promise.reject((_d = result === null || result === void 0 ? void 0 : result.data) === null || _d === void 0 ? void 0 : _d.rejected);
            }
            catch (e) {
                console.log('error', e);
                return ({ error: e });
            }
        });
    }
}
exports.RunnerController = RunnerController;
//# sourceMappingURL=runner-controller.js.map