"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const index_1 = __importDefault(require("./imports/router/index"));
const jwt_1 = __importDefault(require("./imports/router/jwt"));
const guest_1 = __importDefault(require("./imports/router/guest"));
const packager_1 = __importDefault(require("./imports/router/packager"));
const axios_1 = __importDefault(require("axios"));
const http_1 = __importDefault(require("http"));
const http_proxy_middleware_1 = require("http-proxy-middleware");
const graphql_playground_middleware_express_1 = __importDefault(require("graphql-playground-middleware-express"));
const DEEPLINKS_HASURA_PATH = process.env.DEEPLINKS_HASURA_PATH || 'localhost:8080';
const DEEPLINKS_HASURA_SSL = process.env.DEEPLINKS_HASURA_SSL || 0;
const DEEPLINKS_HASURA_SECRET = process.env.DEEPLINKS_HASURA_SECRET || 'myadminsecretkey';
const app = (0, express_1.default)();
const httpServer = http_1.default.createServer(app);
app.get('/gql', (0, graphql_playground_middleware_express_1.default)({
    tabs: [{
            endpoint: '/gql',
            query: `query MyQuery {
      links(limit: 1) {
        id
      }
    }`,
            headers: {
                Authorization: 'Bearer TOKEN',
            },
        }],
}));
app.use('/gql', (0, http_proxy_middleware_1.createProxyMiddleware)({
    target: `http${DEEPLINKS_HASURA_SSL === '1' ? 's' : ''}://${DEEPLINKS_HASURA_PATH}`,
    changeOrigin: true,
    ws: true,
    pathRewrite: {
        "/gql": "/v1/graphql",
    },
}));
app.use(express_1.default.json());
app.use('/', index_1.default);
const start = () => __awaiter(void 0, void 0, void 0, function* () {
    const jwtServer = (0, jwt_1.default)(httpServer);
    const guestServer = (0, guest_1.default)(httpServer);
    const packagerServer = (0, packager_1.default)(httpServer);
    yield jwtServer.start();
    yield guestServer.start();
    yield packagerServer.start();
    jwtServer.applyMiddleware({ path: '/api/jwt', app });
    guestServer.applyMiddleware({ path: '/api/guest', app });
    packagerServer.applyMiddleware({ path: '/api/packager', app });
    yield new Promise(resolve => httpServer.listen({ port: process.env.PORT }, resolve));
    console.log(`Hello bugfixers! Listening ${process.env.PORT} port`);
    try {
        yield (0, axios_1.default)({
            method: 'post',
            url: `http${DEEPLINKS_HASURA_SSL === '1' ? 's' : ''}://${DEEPLINKS_HASURA_PATH}/v1/metadata`,
            headers: { 'x-hasura-admin-secret': DEEPLINKS_HASURA_SECRET, 'Content-Type': 'application/json' },
            data: { type: 'reload_metadata', args: {} }
        });
    }
    catch (e) {
        console.error(e);
    }
});
start();
//# sourceMappingURL=index.js.map